import { useRef, useEffect } from 'react';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { mergeProps } from '../utils/mergeProps.js';
import '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import { useTabbable } from '../Tabbable/Tabbable.js';
import { a as removeIndexFromArray } from '../chunk-29332083.js';
import '../chunk-459cc222.js';
import { useCheckboxState } from './CheckboxState.js';

function useCheckbox(options, htmlProps) {
  if (options === void 0) {
    options = {};
  }

  if (htmlProps === void 0) {
    htmlProps = {};
  }

  options = unstable_useOptions("useCheckbox", options, htmlProps);
  var ref = useRef(null);
  var isBoolean = typeof options.value === "undefined";
  var checked = typeof options.checked !== "undefined" ? options.checked : isBoolean ? Boolean(options.currentValue) : (options.currentValue || []).indexOf(options.value) !== -1;
  useEffect(function () {
    if (ref.current && options.currentValue === "indeterminate") {
      ref.current.indeterminate = true;
    }
  }, [options.currentValue]);
  htmlProps = mergeProps({
    ref: ref,
    checked: checked,
    "aria-checked": options.currentValue === "indeterminate" ? "mixed" : checked,
    value: options.value,
    role: "checkbox",
    type: "checkbox",
    onClick: function onClick(event) {
      if (event.target instanceof HTMLInputElement) return;
      htmlProps.onChange(event);
    },
    onChange: function onChange() {
      if (options.disabled || !options.setValue) return;

      if (isBoolean) {
        options.setValue(!checked);
      } else {
        var array = Array.isArray(options.currentValue) ? options.currentValue : [];
        var index = array.indexOf(options.value);

        if (index === -1) {
          options.setValue([].concat(array, [options.value]));
        } else {
          options.setValue(removeIndexFromArray(array, index));
        }
      }
    }
  }, htmlProps);
  htmlProps = useTabbable(options, htmlProps);
  htmlProps = unstable_useProps("useCheckbox", options, htmlProps);
  return htmlProps;
}
var keys = [].concat(useTabbable.__keys, useCheckboxState.__keys, ["value", "checked"]);
useCheckbox.__keys = keys;
var Checkbox = unstable_createComponent({
  as: "input",
  useHook: useCheckbox
});

export { Checkbox, useCheckbox };
