import { useState, useCallback } from 'react';
import { unstable_useId } from '../utils/useId.js';
import { a as useSealedState } from '../chunk-459cc222.js';

function useHiddenState(initialState) {
  if (initialState === void 0) {
    initialState = {};
  }

  var _useSealedState = useSealedState(initialState),
      _useSealedState$unsta = _useSealedState.unstable_hiddenId,
      hiddenId = _useSealedState$unsta === void 0 ? unstable_useId("hidden-") : _useSealedState$unsta,
      _useSealedState$visib = _useSealedState.visible,
      sealedVisible = _useSealedState$visib === void 0 ? false : _useSealedState$visib;

  var _React$useState = useState(sealedVisible),
      visible = _React$useState[0],
      setVisible = _React$useState[1];

  var show = useCallback(function () {
    return setVisible(true);
  }, []);
  var hide = useCallback(function () {
    return setVisible(false);
  }, []);
  var toggle = useCallback(function () {
    return setVisible(function (v) {
      return !v;
    });
  }, []);
  return {
    unstable_hiddenId: hiddenId,
    visible: visible,
    show: show,
    hide: hide,
    toggle: toggle
  };
}
var keys = ["unstable_hiddenId", "visible", "show", "hide", "toggle"];
useHiddenState.__keys = keys;

export { useHiddenState };
