import { useContext } from 'react';
import '../utils/useId.js';
import { unstable_useCreateElement } from '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { mergeProps } from '../utils/mergeProps.js';
import { b as _objectSpread } from '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import '../Tabbable/Tabbable.js';
import '../chunk-29332083.js';
import '../chunk-459cc222.js';
import { a as warning } from '../chunk-e555b6ea.js';
import 'react-dom';
import '../Portal/Portal.js';
import '../Hidden/HiddenState.js';
import '../Hidden/Hidden.js';
import '../Dialog/Dialog.js';
import 'body-scroll-lock';
import '../chunk-b4d83ecc.js';
import '../Dialog/DialogState.js';
import '../Rover/RoverState.js';
import '../Rover/Rover.js';
import 'popper.js';
import '../Popover/PopoverState.js';
import { usePopover } from '../Popover/Popover.js';
import { useStaticMenu } from './StaticMenu.js';
import { a as MenuContext } from '../chunk-ee62c270.js';
import { useMenuState } from './MenuState.js';

function useMenu(options, htmlProps) {
  if (htmlProps === void 0) {
    htmlProps = {};
  }

  var parent = useContext(MenuContext);
  var parentIsHorizontal = parent && parent.orientation === "horizontal";

  var _options = _objectSpread({
    unstable_autoFocusOnShow: !parent,
    unstable_autoFocusOnHide: !parentIsHorizontal
  }, options, {
    modal: false,
    // We'll handle esc differently
    hideOnEsc: false
  });

  _options = unstable_useOptions("useMenu", _options, htmlProps);
  htmlProps = mergeProps({
    role: "menu",
    onKeyDown: function onKeyDown(event) {
      if (event.key === "Escape" && _options.hide) {
        // Only stop propagtion if there's no parent menu
        // Otherwise, pressing Esc should close all menus
        if (!parent) {
          event.stopPropagation();
        }

        _options.hide();
      }
    }
  }, htmlProps);
  htmlProps = useStaticMenu(_options, htmlProps);
  htmlProps = usePopover(_options, htmlProps);
  htmlProps = unstable_useProps("useMenu", _options, htmlProps);
  return htmlProps;
}
var keys = [].concat(usePopover.__keys, useStaticMenu.__keys, useMenuState.__keys);
useMenu.__keys = keys;
var Menu = unstable_createComponent({
  as: "div",
  useHook: useMenu,
  useCreateElement: function useCreateElement(type, props, children) {
    process.env.NODE_ENV !== "production" ? warning(!props["aria-label"] && !props["aria-labelledby"], "You should provide either `aria-label` or `aria-labelledby` props.\nSee https://www.w3.org/TR/wai-aria-practices-1.1/#wai-aria-roles-states-and-properties-13", "Menu") : void 0;
    return unstable_useCreateElement(type, props, children);
  }
});

export { Menu, useMenu };
