import { useContext, useRef } from 'react';
import '../utils/useId.js';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { mergeProps } from '../utils/mergeProps.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import '../Tabbable/Tabbable.js';
import '../Button/Button.js';
import '../chunk-459cc222.js';
import '../chunk-e555b6ea.js';
import '../Hidden/HiddenState.js';
import '../chunk-b4d83ecc.js';
import '../Dialog/DialogState.js';
import '../Hidden/HiddenDisclosure.js';
import '../Dialog/DialogDisclosure.js';
import '../Rover/RoverState.js';
import '../Rover/Rover.js';
import 'popper.js';
import '../Popover/PopoverState.js';
import { a as MenuContext } from '../chunk-ee62c270.js';
import { useMenuState } from './MenuState.js';
import { usePopoverDisclosure } from '../Popover/PopoverDisclosure.js';

function useMenuDisclosure(options, _ref) {
  if (_ref === void 0) {
    _ref = {};
  }

  var _ref2 = _ref,
      _onKeyDown = _ref2.onKeyDown,
      htmlProps = _objectWithoutPropertiesLoose(_ref2, ["onKeyDown"]);

  var parent = useContext(MenuContext);
  var ref = useRef(null);
  options = unstable_useOptions("useMenuDisclosure", options, htmlProps);
  var dir = options.placement ? options.placement.split("-")[0] : undefined;
  htmlProps = mergeProps({
    ref: ref,
    "aria-haspopup": "menu",
    onFocus: function onFocus() {
      if (parent && parent.orientation === "horizontal") {
        options.show();
      }
    },
    onMouseOver: function onMouseOver() {
      if (!parent || !options.placement) return;
      var parentIsHorizontal = parent.orientation === "horizontal";

      if (!parentIsHorizontal) {
        setTimeout(function () {
          if (ref.current && ref.current.contains(document.activeElement)) {
            options.show();
            ref.current.focus();
          }
        }, 200);
      } else if (ref.current) {
        var parentMenu = ref.current.closest("[role=menu],[role=menubar]");
        var subjacentOpenMenu = parentMenu && parentMenu.querySelector("[role=menu][aria-hidden=false]");

        if (subjacentOpenMenu) {
          ref.current.focus();
        }
      }
    },
    onKeyDown: function onKeyDown(event) {
      var keyMap = {
        Escape: options.hide,
        Enter: parent && options.unstable_first,
        " ": parent && options.unstable_first,
        ArrowUp: dir === "top" || dir === "bottom" ? options.unstable_last : false,
        ArrowRight: dir === "right" && options.unstable_first,
        ArrowDown: dir === "bottom" || dir === "top" ? options.unstable_first : false,
        ArrowLeft: dir === "left" && options.unstable_first
      };

      if (event.key in keyMap) {
        var key = event.key;
        var action = keyMap[key];

        if (typeof action === "function") {
          event.preventDefault();
          options.show();
          action(); // Prevent onKeyDown from being called twice for the same keys.

          return;
        }
      }

      if (_onKeyDown) {
        _onKeyDown(event);
      }
    }
  }, htmlProps);
  htmlProps = usePopoverDisclosure(_objectSpread({}, options, {
    toggle: parent ? options.show : options.toggle
  }), htmlProps);
  htmlProps = unstable_useProps("useMenuDisclosure", options, htmlProps);
  return htmlProps;
}
var keys = [].concat(usePopoverDisclosure.__keys, useMenuState.__keys);
useMenuDisclosure.__keys = keys;
var MenuDisclosure = unstable_createComponent({
  as: "button",
  useHook: useMenuDisclosure
});

export { MenuDisclosure, useMenuDisclosure };
