import { useContext, useRef, useMemo, createElement } from 'react';
import '../utils/useId.js';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { mergeProps } from '../utils/mergeProps.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import '../Tabbable/Tabbable.js';
import '../chunk-459cc222.js';
import '../chunk-e555b6ea.js';
import '../Hidden/HiddenState.js';
import '../chunk-b4d83ecc.js';
import '../Dialog/DialogState.js';
import '../Rover/RoverState.js';
import { useRover } from '../Rover/Rover.js';
import 'popper.js';
import '../Popover/PopoverState.js';
import { a as MenuContext } from '../chunk-ee62c270.js';
import { useMenuState } from './MenuState.js';

function useMenuItem(options, _ref) {
  if (_ref === void 0) {
    _ref = {};
  }

  var _ref2 = _ref,
      children = _ref2.children,
      _onKeyDown = _ref2.onKeyDown,
      htmlProps = _objectWithoutPropertiesLoose(_ref2, ["children", "onKeyDown"]);

  var parent = useContext(MenuContext);
  var ref = useRef(null);
  options = unstable_useOptions("useMenuItem", options, htmlProps);
  var providerValue = useMemo(function () {
    return {
      orientation: options.orientation,
      unstable_next: options.unstable_next,
      unstable_previous: options.unstable_previous,
      parent: parent
    };
  }, [options.orientation, options.unstable_next, options.unstable_previous, parent]);
  htmlProps = mergeProps({
    ref: ref,
    role: "menuitem",
    onMouseOver: function onMouseOver() {
      if (options.orientation !== "horizontal" && ref.current) {
        ref.current.focus();
      }
    },
    onKeyDown: function onKeyDown(event) {
      var _options = options,
          hide = _options.hide,
          placement = _options.placement;
      if (!parent || !hide || !placement) return;
      var horizontalParent = parent;

      while (horizontalParent && horizontalParent.orientation !== "horizontal") {
        horizontalParent = horizontalParent.parent;
      }

      var _placement$split = placement.split("-"),
          dir = _placement$split[0];

      var keyMap = {
        ArrowRight: horizontalParent && dir !== "left" ? function () {
          hide();
          horizontalParent.unstable_next();
        } : dir === "left" && hide,
        ArrowLeft: horizontalParent && dir !== "right" ? function () {
          hide();
          horizontalParent.unstable_previous();
        } : dir === "right" && hide
      };

      if (event.key in keyMap) {
        var key = event.key;
        var action = keyMap[key];

        if (typeof action === "function") {
          event.preventDefault();
          action(); // Prevent onKeyDown from being called twice for the same keys.

          return;
        }
      }

      if (_onKeyDown) {
        _onKeyDown(event);
      }
    }
  }, htmlProps);
  htmlProps = useRover(options, htmlProps);
  htmlProps = unstable_useProps("useMenuItem", options, htmlProps);
  return _objectSpread({}, htmlProps, {
    children: typeof children === "function" ? function (props) {
      return createElement(MenuContext.Provider, {
        value: providerValue
      }, children(props));
    } : children
  });
}
var keys = [].concat(useRover.__keys, useMenuState.__keys);
useMenuItem.__keys = keys;
var MenuItem = unstable_createComponent({
  as: "button",
  useHook: useMenuItem
});

export { MenuItem, useMenuItem };
