import { useRef, useEffect } from 'react';
import { unstable_useId } from '../utils/useId.js';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { mergeProps } from '../utils/mergeProps.js';
import { a as _objectWithoutPropertiesLoose } from '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import { useTabbable } from '../Tabbable/Tabbable.js';
import '../chunk-459cc222.js';
import '../chunk-e555b6ea.js';
import { a as useUpdateEffect } from '../chunk-b4d83ecc.js';
import { useRoverState } from './RoverState.js';

function useRover(options, _ref) {
  if (_ref === void 0) {
    _ref = {};
  }

  var _ref2 = _ref,
      tabIndex = _ref2.tabIndex,
      _onKeyDown = _ref2.onKeyDown,
      htmlProps = _objectWithoutPropertiesLoose(_ref2, ["tabIndex", "onKeyDown"]);

  options = unstable_useOptions("useRover", options, htmlProps);
  var ref = useRef(null);
  var id = unstable_useId("rover-");
  var stopId = options.stopId || htmlProps.id || id;
  var reallyDisabled = options.disabled && !options.unstable_focusable;
  var noFocused = options.unstable_currentId == null;
  var focused = options.unstable_currentId === stopId;
  var isFirst = options.unstable_stops[0] && options.unstable_stops[0].id === stopId;
  var shouldTabIndex = focused || isFirst && noFocused;
  useEffect(function () {
    if (reallyDisabled) return undefined;
    options.unstable_register(stopId, ref);
    return function () {
      return options.unstable_unregister(stopId);
    };
  }, [stopId, reallyDisabled, options.unstable_register, options.unstable_unregister]);
  useUpdateEffect(function () {
    if (ref.current && focused) {
      ref.current.focus();
    }
  }, [focused]);
  htmlProps = mergeProps({
    ref: ref,
    id: stopId,
    tabIndex: shouldTabIndex ? tabIndex : -1,
    onFocus: function onFocus() {
      return options.unstable_move(stopId);
    },
    onKeyDown: function onKeyDown(event) {
      var _options = options,
          orientation = _options.orientation;
      var keyMap = {
        ArrowUp: orientation !== "horizontal" && options.unstable_previous,
        ArrowRight: orientation !== "vertical" && options.unstable_next,
        ArrowDown: orientation !== "horizontal" && options.unstable_next,
        ArrowLeft: orientation !== "vertical" && options.unstable_previous,
        Home: options.unstable_first,
        End: options.unstable_last,
        PageUp: options.unstable_first,
        PageDown: options.unstable_last
      };

      if (event.key in keyMap) {
        var key = event.key;
        var action = keyMap[key];

        if (action) {
          event.preventDefault();
          action(); // Prevent onKeyDown from being called twice for the same keys.

          return;
        }
      }

      if (_onKeyDown) {
        _onKeyDown(event);
      }
    }
  }, htmlProps);
  htmlProps = useTabbable(options, htmlProps);
  htmlProps = unstable_useProps("useRover", options, htmlProps);
  return htmlProps;
}
var keys = [].concat(useTabbable.__keys, useRoverState.__keys, ["stopId"]);
useRover.__keys = keys;
var Rover = unstable_createComponent({
  as: "button",
  useHook: useRover
});

export { Rover, useRover };
