import 'react';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { mergeProps } from '../utils/mergeProps.js';
import { a as _objectWithoutPropertiesLoose, b as _objectSpread } from '../chunk-397c8002.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../system/useToken.js';
import { unstable_useProps } from '../system/useProps.js';
import { unstable_useOptions } from '../system/useOptions.js';
import { useBox } from '../Box/Box.js';
import { a as useLiveRef } from '../chunk-6e18ef4b.js';

function isNativeTabbable(element) {
  if (element instanceof HTMLButtonElement) return true;
  if (element instanceof HTMLInputElement) return true;
  if (element instanceof HTMLSelectElement) return true;
  if (element instanceof HTMLTextAreaElement) return true;
  if (element instanceof HTMLAnchorElement) return true;
  if (element instanceof HTMLAudioElement) return true;
  if (element instanceof HTMLVideoElement) return true;
  return false;
}

function useTabbable(_ref2, _ref) {
  if (_ref2 === void 0) {
    _ref2 = {};
  }

  if (_ref === void 0) {
    _ref = {};
  }

  var _ref3 = _ref2,
      _ref3$unstable_clickK = _ref3.unstable_clickKeys,
      unstable_clickKeys = _ref3$unstable_clickK === void 0 ? [" "] : _ref3$unstable_clickK,
      options = _objectWithoutPropertiesLoose(_ref3, ["unstable_clickKeys"]);

  var _ref4 = _ref,
      _ref4$tabIndex = _ref4.tabIndex,
      tabIndex = _ref4$tabIndex === void 0 ? 0 : _ref4$tabIndex,
      _onClick = _ref4.onClick,
      _onMouseOver = _ref4.onMouseOver,
      disabled = _ref4.disabled,
      htmlProps = _objectWithoutPropertiesLoose(_ref4, ["tabIndex", "onClick", "onMouseOver", "disabled"]);

  var _options = _objectSpread({
    unstable_clickKeys: unstable_clickKeys,
    disabled: disabled
  }, options);

  _options = unstable_useOptions("useTabbable", _options, htmlProps);
  var clickKeysRef = useLiveRef(_options.unstable_clickKeys);
  var reallyDisabled = _options.disabled && !_options.unstable_focusable;
  htmlProps = mergeProps({
    disabled: reallyDisabled,
    tabIndex: reallyDisabled ? undefined : tabIndex,
    "aria-disabled": _options.disabled,
    onClick: function onClick(event) {
      if (_options.disabled) {
        event.stopPropagation();
        event.preventDefault();
      } else if (_onClick) {
        _onClick(event);
      }
    },
    onMouseOver: function onMouseOver(event) {
      if (_options.disabled) {
        event.stopPropagation();
        event.preventDefault();
      } else if (_onMouseOver) {
        _onMouseOver(event);
      }
    },
    onKeyDown: function onKeyDown(event) {
      if (isNativeTabbable(event.target) || _options.disabled) return;

      if (clickKeysRef.current && clickKeysRef.current.indexOf(event.key) !== -1) {
        event.preventDefault();
        event.target.dispatchEvent(new MouseEvent("click", {
          view: window,
          bubbles: true,
          cancelable: false
        }));
      }
    }
  }, htmlProps);
  htmlProps = useBox(_options, htmlProps);
  htmlProps = unstable_useProps("useTabbable", _options, htmlProps);
  return htmlProps;
}
var keys = [].concat(useBox.__keys, ["disabled", "unstable_focusable", "unstable_clickKeys"]);
useTabbable.__keys = keys;
var Tabbable = unstable_createComponent({
  as: "button",
  useHook: useTabbable
});

export { Tabbable, useTabbable };
