'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useId.js');
var useCreateElement = require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../chunk-8e8e1bad.js');
require('../chunk-2138b837.js');
var mergeProps = require('../utils/mergeProps.js');
var __chunk_3 = require('../chunk-37dea005.js');
var createComponent = require('../utils/createComponent.js');
require('../system/useToken.js');
var useProps = require('../system/useProps.js');
var useOptions = require('../system/useOptions.js');
require('../Box/Box.js');
var __chunk_4 = require('../chunk-9263130a.js');
var __chunk_5 = require('../chunk-d4f443f9.js');
require('../chunk-eb7a57a2.js');
var __chunk_7 = require('../chunk-ebc3ed80.js');
require('react-dom');
var Portal = require('../Portal/Portal.js');
require('../Hidden/HiddenState.js');
var Hidden = require('../Hidden/Hidden.js');
var bodyScrollLock = require('body-scroll-lock');
var __chunk_8 = require('../chunk-5a603fe6.js');
var DialogState = require('./DialogState.js');

function useDisclosureRef(options) {
  var disclosureRef = React.useRef(null);
  React.useEffect(function () {
    if (!options.visible) return;
    var selector = "[aria-controls~=\"" + options.unstable_hiddenId + "\"]";
    disclosureRef.current = document.querySelector(selector);
  }, [options.unstable_hiddenId, options.visible]);
  return disclosureRef;
}

function usePreventBodyScroll(targetRef, options) {
  var shouldPrevent = !options.modal ? false : Boolean(options.preventBodyScroll && options.visible);
  React.useEffect(function () {
    var element = targetRef.current;
    if (!element || !shouldPrevent) return undefined;
    bodyScrollLock.disableBodyScroll(element);
    return function () {
      return bodyScrollLock.enableBodyScroll(element);
    };
  }, [targetRef, shouldPrevent]);
}

var selector = "input, select, textarea, a[href], button, [tabindex], audio[controls], video[controls], [contenteditable]:not([contenteditable=false])";

function isHTMLElement(element) {
  return element instanceof HTMLElement;
}

function isDisabled(element) {
  return Boolean(element.disabled);
}

function hasTabIndex(element) {
  return element.hasAttribute("tabindex");
}

function hasNegativeTabIndex(element) {
  return hasTabIndex(element) && element.tabIndex < 0;
}

function isHidden(element) {
  if (element.parentElement && isHidden(element.parentElement)) return true;
  return element.hidden;
}

function isContentEditable(element) {
  var value = element.getAttribute("contenteditable");
  return value !== "false" && value != null;
}

function isFocusable(element) {
  if (!isHTMLElement(element)) return false;
  if (isHidden(element)) return false;
  if (isDisabled(element)) return false;
  var localName = element.localName;
  var focusableTags = ["input", "select", "textarea", "button"];
  if (focusableTags.indexOf(localName) >= 0) return true;
  var others = {
    a: function a() {
      return element.hasAttribute("href");
    },
    audio: function audio() {
      return element.hasAttribute("controls");
    },
    video: function video() {
      return element.hasAttribute("controls");
    }
  };

  if (localName in others) {
    return others[localName]();
  }

  if (isContentEditable(element)) return true;
  return hasTabIndex(element);
}
function isTabbable(element) {
  if (!isHTMLElement(element)) return false;
  if (!isFocusable(element)) return false;
  if (hasNegativeTabIndex(element)) return false;
  return true;
}
function getAllTabbableIn(container, fallbackToFocusable) {
  var allFocusable = Array.from(container.querySelectorAll(selector));
  var allTabbable = allFocusable.filter(isTabbable);

  if (!allTabbable.length && fallbackToFocusable) {
    return allFocusable;
  }

  return allTabbable;
}
function getFirstTabbableIn(container, fallbackToFocusable) {
  var _getAllTabbableIn = getAllTabbableIn(container, fallbackToFocusable),
      first = _getAllTabbableIn[0];

  return first || null;
}
function getLastTabbableIn(container, fallbackToFocusable) {
  var allTabbable = getAllTabbableIn(container, fallbackToFocusable);
  return allTabbable[allTabbable.length - 1] || null;
}

function useFocusOnShow(dialogRef, nestedDialogs, options) {
  var initialFocusRef = options.unstable_initialFocusRef;
  var shouldFocus = options.visible && options.unstable_autoFocusOnShow;
  __chunk_8.useUpdateEffect(function () {
    var dialog = dialogRef.current; // If there're nested open dialogs, let them handle focus

    if (!shouldFocus || !dialog || nestedDialogs.find(function (child) {
      return Boolean(child.current && !child.current.hidden);
    })) {
      return;
    }

    if (initialFocusRef && initialFocusRef.current) {
      initialFocusRef.current.focus();
    } else {
      var tabbable = getFirstTabbableIn(dialog, true);

      if (tabbable) {
        tabbable.focus();
      } else {
        dialog.focus();
      }
    }
  }, [dialogRef, initialFocusRef, shouldFocus, nestedDialogs]);
}

function usePortalRef(dialogRef, options) {
  var portalRef = React.useRef(null);
  React.useEffect(function () {
    var dialog = dialogRef.current;
    if (!dialog || !options.visible) return;
    portalRef.current = dialog.closest(Portal.Portal.__selector);
  }, [dialogRef, options.visible]);
  return portalRef;
}

function hasNestedOpenModals(portal) {
  return portal.querySelectorAll("[role=dialog][aria-modal=true][aria-hidden=false]").length > 1;
}

var focusTrapClassName = "__reakit-focus-trap";
function isFocusTrap(element) {
  return element.classList.contains(focusTrapClassName);
}
function useFocusTrap(dialogRef, portalRef, options) {
  var shouldTrap = options.visible && options.modal;
  var beforeElement = React.useRef(null);
  var afterElement = React.useRef(null); // Create before and after elements
  // https://github.com/w3c/aria-practices/issues/545

  React.useEffect(function () {
    var portal = portalRef.current;
    if (!portal || !shouldTrap) return undefined;

    if (!beforeElement.current) {
      beforeElement.current = document.createElement("div");
      beforeElement.current.className = focusTrapClassName;
      beforeElement.current.tabIndex = 0;
      beforeElement.current.style.position = "fixed";
      beforeElement.current.setAttribute("aria-hidden", "true");
    }

    if (!afterElement.current) {
      afterElement.current = beforeElement.current.cloneNode();
    }

    portal.insertAdjacentElement("beforebegin", beforeElement.current);
    portal.insertAdjacentElement("afterend", afterElement.current);
    return function () {
      if (beforeElement.current) beforeElement.current.remove();
      if (afterElement.current) afterElement.current.remove();
    };
  }, [portalRef, shouldTrap]); // Focus trap

  React.useEffect(function () {
    var before = beforeElement.current;
    var after = afterElement.current;
    if (!shouldTrap || !before || !after) return undefined;

    var handleFocus = function handleFocus(e) {
      var portal = portalRef.current;
      var dialog = dialogRef.current;
      if (!portal || !dialog || hasNestedOpenModals(portal)) return;
      e.preventDefault();
      var isAfter = e.target === after;
      var tabbable = isAfter ? getFirstTabbableIn(dialog) : getLastTabbableIn(dialog);

      if (tabbable) {
        tabbable.focus();
      } else {
        // fallback to dialog
        dialog.focus();
      }
    };

    before.addEventListener("focus", handleFocus);
    after.addEventListener("focus", handleFocus);
    return function () {
      before.removeEventListener("focus", handleFocus);
      after.removeEventListener("focus", handleFocus);
    };
  }, [dialogRef, portalRef, shouldTrap]); // Click trap

  React.useEffect(function () {
    if (!shouldTrap) return undefined;

    var handleClick = function handleClick() {
      var dialog = dialogRef.current;
      var portal = portalRef.current;
      if (!dialog || !portal || hasNestedOpenModals(portal)) return;

      if (!portal.contains(document.activeElement)) {
        dialog.focus();
      }
    };

    document.addEventListener("click", handleClick);
    return function () {
      document.removeEventListener("click", handleClick);
    };
  }, [dialogRef, portalRef, shouldTrap]);
}

function useFocusOnHide(dialogRef, disclosureRef, options) {
  var finalFocusRef = options.unstable_finalFocusRef || disclosureRef;
  var shouldFocus = options.unstable_autoFocusOnHide && !options.visible;
  __chunk_8.useUpdateEffect(function () {
    var dialog = dialogRef.current;
    if (!dialog || !shouldFocus) return; // Hide was triggered by a click/focus on a tabbable element outside
    // the dialog. We won't change focus then.

    if (document.activeElement && !dialog.contains(document.activeElement) && isTabbable(document.activeElement)) {
      return;
    }

    if (finalFocusRef && finalFocusRef.current) {
      finalFocusRef.current.focus();
    }
  }, [dialogRef, finalFocusRef, shouldFocus]);
}

function removeItemFromArray(array, item) {
  var idx = array.indexOf(item);
  return __chunk_5.removeIndexFromArray(array, idx);
}

var DialogContext = React.createContext({});
function useNestedDialogs(dialogRef, options) {
  var context = React.useContext(DialogContext);

  var _React$useState = React.useState([]),
      dialogs = _React$useState[0],
      setDialogs = _React$useState[1];

  var addDialog = React.useCallback(function (ref) {
    if (context.addDialog) {
      context.addDialog(ref);
    }

    setDialogs(function (refs) {
      return [].concat(refs, [ref]);
    });
  }, [context.addDialog]);
  var removeDialog = React.useCallback(function (ref) {
    if (context.removeDialog) {
      context.removeDialog(ref);
    }

    setDialogs(function (refs) {
      return removeItemFromArray(refs, ref);
    });
  }, [context.removeDialog]); // If it's a nested dialog, add it to context

  React.useEffect(function () {
    if (!context.addDialog) return undefined;
    context.addDialog(dialogRef);
    return function () {
      if (context.removeDialog) {
        context.removeDialog(dialogRef);
      }
    };
  }, [dialogRef, context.addDialog, context.removeDialog]); // Close all nested dialogs when parent dialog closes

  React.useEffect(function () {
    if (context.visible === false && options.visible && options.hide) {
      options.hide();
    }
  }, [context.visible, options.visible, options.hide]); // Provider

  var providerValue = React.useMemo(function () {
    return {
      visible: options.visible,
      addDialog: addDialog,
      removeDialog: removeDialog
    };
  }, [options.visible, addDialog, removeDialog]);

  var wrapChildren = function wrapChildren(children) {
    return React.createElement(DialogContext.Provider, {
      value: providerValue
    }, children);
  };

  return {
    dialogs: dialogs,
    wrapChildren: wrapChildren
  };
}

function useEventListenerOutside(targetRef, disclosureRef, nestedDialogs, event, listener, shouldListen) {
  var listenerRef = __chunk_4.useLiveRef(listener);
  React.useEffect(function () {
    if (!shouldListen) return undefined;

    var handleEvent = function handleEvent(e) {
      if (!listenerRef.current) return;
      var element = targetRef.current;
      var disclosure = disclosureRef.current;
      var target = e.target; // Click inside

      if (!element || element.contains(target)) return; // Click on disclosure

      if (disclosure && disclosure.contains(target)) return; // Click inside a nested dialog or focus trap

      if (isFocusTrap(target) || nestedDialogs.find(function (dialog) {
        return Boolean(dialog.current && dialog.current.contains(target));
      })) {
        return;
      }

      listenerRef.current(e);
    };

    document.addEventListener(event, handleEvent, true);
    return function () {
      document.removeEventListener(event, handleEvent, true);
    };
  }, [targetRef, event, shouldListen, nestedDialogs]);
}

function useHideOnClickOutside(portalRef, disclosureRef, nestedDialogs, options) {
  return useEventListenerOutside(portalRef, disclosureRef, nestedDialogs, "click", options.hide, options.visible && options.hideOnClickOutside);
}

function useHideOnFocusOutside(dialogRef, disclosureRef, nestedDialogs, options) {
  return useEventListenerOutside(dialogRef, disclosureRef, nestedDialogs, "focus", options.hide, options.visible && options.hideOnClickOutside && !options.modal);
}

function useDialog(_ref2, _ref) {
  if (_ref === void 0) {
    _ref = {};
  }

  var _ref2$modal = _ref2.modal,
      modal = _ref2$modal === void 0 ? true : _ref2$modal,
      _ref2$hideOnEsc = _ref2.hideOnEsc,
      hideOnEsc = _ref2$hideOnEsc === void 0 ? true : _ref2$hideOnEsc,
      _ref2$hideOnClickOuts = _ref2.hideOnClickOutside,
      hideOnClickOutside = _ref2$hideOnClickOuts === void 0 ? true : _ref2$hideOnClickOuts,
      _ref2$preventBodyScro = _ref2.preventBodyScroll,
      preventBodyScroll = _ref2$preventBodyScro === void 0 ? true : _ref2$preventBodyScro,
      _ref2$unstable_autoFo = _ref2.unstable_autoFocusOnShow,
      unstable_autoFocusOnShow = _ref2$unstable_autoFo === void 0 ? true : _ref2$unstable_autoFo,
      _ref2$unstable_autoFo2 = _ref2.unstable_autoFocusOnHide,
      unstable_autoFocusOnHide = _ref2$unstable_autoFo2 === void 0 ? true : _ref2$unstable_autoFo2,
      options = __chunk_3._objectWithoutPropertiesLoose(_ref2, ["modal", "hideOnEsc", "hideOnClickOutside", "preventBodyScroll", "unstable_autoFocusOnShow", "unstable_autoFocusOnHide"]);

  var _ref3 = _ref,
      children = _ref3.children,
      htmlProps = __chunk_3._objectWithoutPropertiesLoose(_ref3, ["children"]);

  var _options = __chunk_3._objectSpread({
    modal: modal,
    hideOnEsc: hideOnEsc,
    hideOnClickOutside: hideOnClickOutside,
    preventBodyScroll: preventBodyScroll,
    unstable_autoFocusOnShow: unstable_autoFocusOnShow,
    unstable_autoFocusOnHide: unstable_autoFocusOnHide
  }, options);

  _options = useOptions.unstable_useOptions("useDialog", _options, htmlProps);
  var dialog = React.useRef(null);
  var portal = usePortalRef(dialog, _options);
  var disclosure = useDisclosureRef(_options);

  var _useNestedDialogs = useNestedDialogs(dialog, _options),
      dialogs = _useNestedDialogs.dialogs,
      wrapChildren = _useNestedDialogs.wrapChildren;

  usePreventBodyScroll(dialog, _options);
  useFocusTrap(dialog, portal, _options);
  useFocusOnShow(dialog, dialogs, _options);
  useFocusOnHide(dialog, disclosure, _options);
  useHideOnClickOutside(portal, disclosure, dialogs, _options);
  useHideOnFocusOutside(dialog, disclosure, dialogs, _options);
  htmlProps = mergeProps.mergeProps({
    ref: dialog,
    role: "dialog",
    tabIndex: -1,
    "aria-modal": _options.modal,
    "data-dialog": true,
    style: {
      zIndex: 999
    },
    children: wrapChildren(children),
    onKeyDown: function onKeyDown(event) {
      if (event.key === "Escape" && _options.hide && _options.hideOnEsc) {
        event.stopPropagation();

        _options.hide();
      }
    }
  }, // If it's not modal, it doesn't have a portal
  !_options.modal && {
    className: Portal.Portal.__className
  }, htmlProps);
  htmlProps = Hidden.useHidden(_options, htmlProps);
  htmlProps = useProps.unstable_useProps("useDialog", _options, htmlProps);
  return htmlProps;
}
var keys = [].concat(Hidden.useHidden.__keys, DialogState.useDialogState.__keys, ["modal", "hideOnEsc", "hideOnClickOutside", "preventBodyScroll", "unstable_initialFocusRef", "unstable_finalFocusRef", "unstable_autoFocusOnShow", "unstable_autoFocusOnHide"]);
useDialog.__keys = keys;
var Dialog = createComponent.unstable_createComponent({
  as: "div",
  useHook: useDialog,
  useCreateElement: function useCreateElement$1(type, props, children) {
    process.env.NODE_ENV !== "production" ? __chunk_7.warning(!props["aria-label"] && !props["aria-labelledby"], "You should provide either `aria-label` or `aria-labelledby` props.\nSee https://www.w3.org/TR/wai-aria-practices-1.1/#dialog_roles_states_props", "Dialog") : void 0;
    var element = useCreateElement.unstable_useCreateElement(type, props, children);

    if (props["aria-modal"]) {
      return React.createElement(Portal.Portal, null, element);
    }

    return element;
  }
});

exports.Dialog = Dialog;
exports.useDialog = useDialog;
