'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var useId = require('../utils/useId.js');
var __chunk_2 = require('../chunk-2138b837.js');
var __chunk_3 = require('../chunk-37dea005.js');
var __chunk_6 = require('../chunk-eb7a57a2.js');
var __chunk_8 = require('../chunk-5a603fe6.js');
var setAllIn = require('./utils/setAllIn.js');
var getIn = require('./utils/getIn.js');
var setIn = require('./utils/setIn.js');

function isPromise(arg) {
  return Boolean(arg && arg.then);
}

function isEmpty(arg) {
  if (Array.isArray(arg)) return !arg.length;
  if (__chunk_2.isObject(arg)) return !Object.keys(arg).length;
  if (arg == null) return true;
  if (arg === "") return true;
  return false;
}

function filterAllEmpty(object) {
  if (Array.isArray(object)) {
    return object.filter(function (value) {
      if (__chunk_2.isObject(value)) {
        return filterAllEmpty(value);
      }

      return true;
    });
  }

  var result = {};
  var keys = Object.keys(object);

  for (var _i = 0, _keys = keys; _i < _keys.length; _i++) {
    var key = _keys[_i];
    var k = key;
    var value = object[k];
    result[k] = __chunk_2.isObject(value) ? filterAllEmpty(value) : object[k];
  }

  return result;
}

function hasMessages(errors) {
  return __chunk_2.isObject(errors) && !isEmpty(errors);
}

function getMessages(stateMessages, actionMessages) {
  return !isEmpty(actionMessages) ? actionMessages : isEmpty(stateMessages) ? stateMessages : {};
}

function reducer(state, action) {
  switch (action.type) {
    case "reset":
      {
        return __chunk_3._objectSpread({}, state, {
          values: state.initialValues,
          touched: {},
          errors: {},
          messages: {},
          valid: true,
          validating: false,
          submitting: false,
          submitFailed: 0,
          submitSucceed: 0
        });
      }

    case "startValidate":
      {
        return __chunk_3._objectSpread({}, state, {
          validating: true
        });
      }

    case "endValidate":
      {
        return __chunk_3._objectSpread({}, state, {
          validating: false,
          errors: getMessages(state.errors, action.errors),
          messages: getMessages(state.messages, action.messages),
          valid: !hasMessages(action.errors)
        });
      }

    case "startSubmit":
      {
        return __chunk_3._objectSpread({}, state, {
          // @ts-ignore TS bug
          touched: setAllIn.unstable_setAllIn(state.values, true),
          submitting: true
        });
      }

    case "endSubmit":
      {
        var valid = !hasMessages(action.errors);
        return __chunk_3._objectSpread({}, state, {
          valid: valid,
          submitting: false,
          errors: getMessages(state.errors, action.errors),
          messages: getMessages(state.messages, action.messages),
          submitSucceed: valid ? state.submitSucceed + 1 : state.submitSucceed,
          submitFailed: valid ? state.submitFailed : state.submitFailed + 1
        });
      }

    case "update":
      {
        var _name = action.name,
            _value = action.value;
        var nextValue = typeof _value === "function" ? _value(getIn.unstable_getIn(state.values, _name)) : _value;
        return __chunk_3._objectSpread({}, state, {
          values: setIn.unstable_setIn(state.values, _name, nextValue != null ? nextValue : "")
        });
      }

    case "blur":
      {
        return __chunk_3._objectSpread({}, state, {
          touched: setIn.unstable_setIn(state.touched, action.name, true)
        });
      }

    case "push":
      {
        var array = getIn.unstable_getIn(state.values, action.name, []);
        return __chunk_3._objectSpread({}, state, {
          values: setIn.unstable_setIn(state.values, action.name, [].concat(array, [action.value]))
        });
      }

    case "remove":
      {
        var _array = getIn.unstable_getIn(state.values, action.name, []);

        delete _array[action.index];
        return __chunk_3._objectSpread({}, state, {
          values: setIn.unstable_setIn(state.values, action.name, _array)
        });
      }

    default:
      {
        throw new Error();
      }
  }
}

function unstable_useFormState(initialState) {
  if (initialState === void 0) {
    initialState = {};
  }

  var _useSealedState = __chunk_6.useSealedState(initialState),
      _useSealedState$baseI = _useSealedState.baseId,
      baseId = _useSealedState$baseI === void 0 ? useId.unstable_useId("form-") : _useSealedState$baseI,
      _useSealedState$value = _useSealedState.values,
      initialValues = _useSealedState$value === void 0 ? {} : _useSealedState$value,
      _useSealedState$valid = _useSealedState.validateOnBlur,
      validateOnBlur = _useSealedState$valid === void 0 ? true : _useSealedState$valid,
      _useSealedState$valid2 = _useSealedState.validateOnChange,
      validateOnChange = _useSealedState$valid2 === void 0 ? true : _useSealedState$valid2,
      _useSealedState$reset = _useSealedState.resetOnSubmitSucceed,
      resetOnSubmitSucceed = _useSealedState$reset === void 0 ? false : _useSealedState$reset,
      _useSealedState$reset2 = _useSealedState.resetOnUnmount,
      resetOnUnmount = _useSealedState$reset2 === void 0 ? true : _useSealedState$reset2,
      onValidate = _useSealedState.onValidate,
      onSubmit = _useSealedState.onSubmit;

  var _React$useReducer = React.useReducer(reducer, {
    baseId: baseId,
    initialValues: initialValues,
    values: initialValues,
    touched: {},
    errors: {},
    messages: {},
    valid: true,
    validating: false,
    submitting: false,
    submitFailed: 0,
    submitSucceed: 0
  }),
      _React$useReducer$ = _React$useReducer[0],
      _ = _React$useReducer$.initialValues,
      state = __chunk_3._objectWithoutPropertiesLoose(_React$useReducer$, ["initialValues"]),
      dispatch = _React$useReducer[1];

  var validate = React.useCallback(
  /*#__PURE__*/
  function () {
    var _ref = __chunk_3._asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee(vals) {
      var response, messages;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (vals === void 0) {
                vals = state.values;
              }

              _context.prev = 1;

              if (!onValidate) {
                _context.next = 10;
                break;
              }

              response = onValidate(filterAllEmpty(vals));

              if (isPromise(response)) {
                dispatch({
                  type: "startValidate"
                });
              }

              _context.next = 7;
              return response;

            case 7:
              messages = _context.sent;
              dispatch({
                type: "endValidate",
                messages: messages
              });
              return _context.abrupt("return", messages);

            case 10:
              return _context.abrupt("return", undefined);

            case 13:
              _context.prev = 13;
              _context.t0 = _context["catch"](1);
              dispatch({
                type: "endValidate",
                errors: _context.t0
              });
              throw _context.t0;

            case 17:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[1, 13]]);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }(), [state.values, onValidate]);
  __chunk_8.useUpdateEffect(function () {
    if (validateOnChange) {
      validate()["catch"](function () {});
    }
  }, [validate, validateOnChange]);
  React.useEffect(function () {
    if (resetOnUnmount) {
      return function () {
        dispatch({
          type: "reset"
        });
      };
    }

    return undefined;
  }, [resetOnUnmount]);
  return __chunk_3._objectSpread({}, state, {
    values: state.values,
    validate: validate,
    reset: React.useCallback(function () {
      return dispatch({
        type: "reset"
      });
    }, []),
    submit: React.useCallback(
    /*#__PURE__*/
    __chunk_3._asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2() {
      var validateMessages, submitMessages, messages;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.prev = 0;
              dispatch({
                type: "startSubmit"
              });
              _context2.next = 4;
              return validate();

            case 4:
              validateMessages = _context2.sent;

              if (!onSubmit) {
                _context2.next = 14;
                break;
              }

              _context2.next = 8;
              return onSubmit(state.values);

            case 8:
              submitMessages = _context2.sent;
              messages = __chunk_3._objectSpread({}, validateMessages, submitMessages);
              dispatch({
                type: "endSubmit",
                messages: messages
              });

              if (resetOnSubmitSucceed) {
                dispatch({
                  type: "reset"
                });
              }

              _context2.next = 15;
              break;

            case 14:
              dispatch({
                type: "endSubmit",
                messages: validateMessages
              });

            case 15:
              _context2.next = 20;
              break;

            case 17:
              _context2.prev = 17;
              _context2.t0 = _context2["catch"](0);
              dispatch({
                type: "endSubmit",
                errors: _context2.t0
              });

            case 20:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, null, [[0, 17]]);
    })), [validate]),
    update: React.useCallback(function (name, value) {
      return dispatch({
        type: "update",
        name: name,
        value: value
      });
    }, []),
    blur: React.useCallback(function (name) {
      dispatch({
        type: "blur",
        name: name
      });

      if (validateOnBlur) {
        validate()["catch"](function () {});
      }
    }, [validate]),
    push: React.useCallback(function (name, value) {
      return dispatch({
        type: "push",
        name: name,
        value: value
      });
    }, []),
    remove: React.useCallback(function (name, index) {
      return dispatch({
        type: "remove",
        name: name,
        index: index
      });
    }, [])
  });
}
var keys = ["baseId", "values", "touched", "messages", "errors", "validating", "valid", "submitting", "submitSucceed", "submitFailed", "validate", "submit", "reset", "update", "blur", "push", "remove"];
unstable_useFormState.__keys = keys;

exports.unstable_useFormState = unstable_useFormState;
