'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var __chunk_3 = require('../chunk-37dea005.js');
var __chunk_6 = require('../chunk-eb7a57a2.js');
var __chunk_7 = require('../chunk-ebc3ed80.js');

function reducer(state, action) {
  var stops = state.unstable_stops,
      currentId = state.unstable_currentId,
      pastId = state.unstable_pastId,
      loop = state.unstable_loop;

  switch (action.type) {
    case "register":
      {
        var _id = action.id,
            _ref = action.ref;

        if (stops.length === 0) {
          return __chunk_3._objectSpread({}, state, {
            unstable_stops: [{
              id: _id,
              ref: _ref
            }]
          });
        }

        var index = stops.findIndex(function (stop) {
          return stop.id === _id;
        });

        if (index >= 0) {
          return state;
        }

        var afterRefIndex = stops.findIndex(function (stop) {
          if (!stop.ref.current || !_ref.current) return false;
          return Boolean(stop.ref.current.compareDocumentPosition(_ref.current) & Node.DOCUMENT_POSITION_PRECEDING);
        });

        if (afterRefIndex === -1) {
          return __chunk_3._objectSpread({}, state, {
            unstable_stops: [].concat(stops, [{
              id: _id,
              ref: _ref
            }])
          });
        }

        return __chunk_3._objectSpread({}, state, {
          unstable_stops: [].concat(stops.slice(0, afterRefIndex), [{
            id: _id,
            ref: _ref
          }], stops.slice(afterRefIndex))
        });
      }

    case "unregister":
      {
        var _id2 = action.id;
        var nextStops = stops.filter(function (stop) {
          return stop.id !== _id2;
        });

        if (nextStops.length === stops.length) {
          process.env.NODE_ENV !== "production" ? __chunk_7.warning(true, _id2 + " stop is not registered", "RoverState") : void 0;
          return state;
        }

        return __chunk_3._objectSpread({}, state, {
          unstable_stops: nextStops,
          unstable_pastId: pastId && pastId === _id2 ? null : pastId,
          unstable_currentId: currentId && currentId === _id2 ? null : currentId
        });
      }

    case "move":
      {
        var _id3 = action.id;

        var _index = stops.findIndex(function (stop) {
          return stop.id === _id3;
        });

        if (_index === -1 || stops[_index].id === currentId) {
          return state;
        }

        return __chunk_3._objectSpread({}, state, {
          unstable_currentId: stops[_index].id,
          unstable_pastId: currentId
        });
      }

    case "next":
      {
        if (currentId == null) {
          return reducer(state, {
            type: "move",
            id: stops[0] && stops[0].id
          });
        }

        var _index2 = stops.findIndex(function (stop) {
          return stop.id === currentId;
        }); // If loop is truthy, turns [0, currentId, 2, 3] into [currentId, 2, 3, 0]
        // Otherwise turns into [currentId, 2, 3]


        var reorderedStops = [].concat(stops.slice(_index2 + 1), loop ? stops.slice(0, _index2) : []);
        var nextIndex = reorderedStops.findIndex(function (stop) {
          return stop.id === currentId;
        }) + 1;
        return reducer(state, {
          type: "move",
          id: reorderedStops[nextIndex] && reorderedStops[nextIndex].id
        });
      }

    case "previous":
      {
        var nextState = reducer(__chunk_3._objectSpread({}, state, {
          unstable_stops: stops.slice().reverse()
        }), {
          type: "next"
        });
        return __chunk_3._objectSpread({}, state, {
          unstable_currentId: nextState.unstable_currentId,
          unstable_pastId: nextState.unstable_pastId
        });
      }

    case "first":
      {
        var stop = stops[0];
        return reducer(state, {
          type: "move",
          id: stop && stop.id
        });
      }

    case "last":
      {
        var _stop = stops[stops.length - 1];
        return reducer(state, {
          type: "move",
          id: _stop && _stop.id
        });
      }

    case "reset":
      {
        return __chunk_3._objectSpread({}, state, {
          unstable_currentId: null,
          unstable_pastId: null
        });
      }

    case "orientate":
      return __chunk_3._objectSpread({}, state, {
        orientation: action.orientation
      });

    default:
      throw new Error();
  }
}

function useRoverState(initialState) {
  if (initialState === void 0) {
    initialState = {};
  }

  var _useSealedState = __chunk_6.useSealedState(initialState),
      _useSealedState$unsta = _useSealedState.unstable_currentId,
      currentId = _useSealedState$unsta === void 0 ? null : _useSealedState$unsta,
      _useSealedState$unsta2 = _useSealedState.unstable_loop,
      loop = _useSealedState$unsta2 === void 0 ? false : _useSealedState$unsta2,
      sealed = __chunk_3._objectWithoutPropertiesLoose(_useSealedState, ["unstable_currentId", "unstable_loop"]);

  var _React$useReducer = React.useReducer(reducer, __chunk_3._objectSpread({}, sealed, {
    unstable_stops: [],
    unstable_currentId: currentId,
    unstable_pastId: null,
    unstable_loop: loop
  })),
      state = _React$useReducer[0],
      dispatch = _React$useReducer[1];

  return __chunk_3._objectSpread({}, state, {
    unstable_register: React.useCallback(function (id, ref) {
      return dispatch({
        type: "register",
        id: id,
        ref: ref
      });
    }, []),
    unstable_unregister: React.useCallback(function (id) {
      return dispatch({
        type: "unregister",
        id: id
      });
    }, []),
    unstable_move: React.useCallback(function (id) {
      return dispatch({
        type: "move",
        id: id
      });
    }, []),
    unstable_next: React.useCallback(function () {
      return dispatch({
        type: "next"
      });
    }, []),
    unstable_previous: React.useCallback(function () {
      return dispatch({
        type: "previous"
      });
    }, []),
    unstable_first: React.useCallback(function () {
      return dispatch({
        type: "first"
      });
    }, []),
    unstable_last: React.useCallback(function () {
      return dispatch({
        type: "last"
      });
    }, []),
    unstable_reset: React.useCallback(function () {
      return dispatch({
        type: "reset"
      });
    }, []),
    unstable_orientate: React.useCallback(function (o) {
      return dispatch({
        type: "orientate",
        orientation: o
      });
    }, [])
  });
}
var keys = ["orientation", "unstable_stops", "unstable_currentId", "unstable_pastId", "unstable_loop", "unstable_register", "unstable_unregister", "unstable_move", "unstable_next", "unstable_previous", "unstable_first", "unstable_last", "unstable_reset", "unstable_orientate"];
useRoverState.__keys = keys;

exports.useRoverState = useRoverState;
