'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var __chunk_1 = require('../chunk-8e8e1bad.js');
var __chunk_2 = require('../chunk-2138b837.js');

function extractPropFromObjects(objects, prop) {
  var props = [];
  var length = objects.length;

  for (var i = 0; i < length; i += 1) {
    var p = objects[i][prop];
    if (p) props.push(p);
  }

  return props;
}

function mergeFunctionsInObjects(objects) {
  var object = __chunk_1.reduceObjects(objects, function (value) {
    return typeof value === "function";
  });
  var keys = Object.keys(object);
  var result = {};

  for (var _i = 0, _keys = keys; _i < _keys.length; _i++) {
    var key = _keys[_i];
    var fns = object[key];
    result[key] = fns.length === 1 ? fns[0] : fns.reduce(function (lastFn, currFn) {
      return function () {
        lastFn.apply(void 0, arguments);
        return currFn.apply(void 0, arguments);
      };
    });
  }

  return result;
}

function mergeRefsInObjects(objects) {
  var refs = extractPropFromObjects(objects, "ref");
  var length = refs.length;
  if (length === 0) return {};
  if (length === 1) return {
    ref: refs[0]
  };
  return {
    ref: function ref(instance) {
      for (var _iterator = refs, _isArray = Array.isArray(_iterator), _i2 = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i2 >= _iterator.length) break;
          _ref = _iterator[_i2++];
        } else {
          _i2 = _iterator.next();
          if (_i2.done) break;
          _ref = _i2.value;
        }

        var ref = _ref;

        if (typeof ref === "function") {
          ref(instance);
        } else if (ref) {
          ref.current = instance;
        }
      }
    }
  };
}

function mergeClassNamesInObjects(objects) {
  var classNames = extractPropFromObjects(objects, "className");
  if (classNames.length === 0) return {};
  return {
    className: classNames.join(" ")
  };
}

function mergeStylesInObjects(objects) {
  var styles = extractPropFromObjects(objects, "style");
  if (styles.length === 0) return {};
  if (styles.length === 1) return {
    style: styles[0]
  };
  return {
    style: Object.assign.apply(Object, [{}].concat(styles))
  };
}

function mergeProps() {
  for (var _len = arguments.length, objects = new Array(_len), _key = 0; _key < _len; _key++) {
    objects[_key] = arguments[_key];
  }

  objects = objects.filter(__chunk_2.isObject);
  return Object.assign.apply(Object, [{}].concat(objects, [mergeFunctionsInObjects(objects), mergeRefsInObjects(objects), mergeClassNamesInObjects(objects), mergeStylesInObjects(objects)]));
}

exports.mergeProps = mergeProps;
