import * as React from "react";
import { Placement } from "popper.js";
import { SealedInitialState } from "../__utils/useSealedState";
import { DialogState, DialogActions, DialogInitialState } from "../Dialog/DialogState";
export declare type PopoverState = DialogState & {
    /**
     * The reference element.
     */
    unstable_referenceRef: React.RefObject<HTMLElement | null>;
    /**
     * The popover element.
     */
    unstable_popoverRef: React.RefObject<HTMLElement | null>;
    /**
     * The arrow element.
     */
    unstable_arrowRef: React.RefObject<HTMLElement | null>;
    /**
     * Popover styles.
     */
    unstable_popoverStyles: Partial<CSSStyleDeclaration>;
    /**
     * Arrow styles.
     */
    unstable_arrowStyles: Partial<CSSStyleDeclaration>;
    /**
     * `placement` passed to the hook.
     */
    unstable_originalPlacement: Placement;
    /**
     * Actual `placement`.
     * @default "bottom"
     */
    placement: Placement;
};
export declare type PopoverActions = DialogActions & {
    /**
     * Change the `placement` state.
     */
    place: React.Dispatch<React.SetStateAction<Placement>>;
};
export declare type PopoverInitialState = DialogInitialState & Partial<Pick<PopoverState, "placement">> & {
    /**
     * Whether or not flip the popover.
     * @default true
     */
    unstable_flip?: boolean;
    /**
     * Whether or not shift the popover.
     * @default true
     */
    unstable_shift?: boolean;
    /**
     * Offset between the reference and the popover.
     * @default 12
     */
    unstable_gutter?: number;
};
export declare type PopoverStateReturn = PopoverState & PopoverActions;
export declare function usePopoverState(initialState?: SealedInitialState<PopoverInitialState>): PopoverStateReturn;
export declare namespace usePopoverState {
    var __keys: ("show" | "hide" | "unstable_hiddenId" | "visible" | "unstable_referenceRef" | "unstable_popoverRef" | "unstable_arrowRef" | "unstable_popoverStyles" | "unstable_arrowStyles" | "unstable_originalPlacement" | "placement" | "toggle" | "place")[];
}
