import * as React from "react";
import { SealedInitialState } from "../__utils/useSealedState";
declare type Stop = {
    id: string;
    ref: React.RefObject<HTMLElement>;
};
export declare type RoverState = {
    /**
     * Defines the orientation of the rover list.
     */
    orientation?: "horizontal" | "vertical";
    /**
     * A list of element refs and IDs of the roving items.
     */
    unstable_stops: Stop[];
    /**
     * The current focused element ID.
     */
    unstable_currentId: Stop["id"] | null;
    /**
     * The last focused element ID.
     */
    unstable_pastId: Stop["id"] | null;
    /**
     * If enabled, the next item after the last one will be the first one.
     */
    unstable_loop: boolean;
};
export declare type RoverActions = {
    /**
     * Registers the element ID and ref in the roving tab index list.
     */
    unstable_register: (id: Stop["id"], ref: Stop["ref"]) => void;
    /**
     * Unregisters the roving item.
     */
    unstable_unregister: (id: Stop["id"]) => void;
    /**
     * Moves focus onto a given element ID.
     */
    unstable_move: (id: Stop["id"]) => void;
    /**
     * Moves focus onto the next element.
     */
    unstable_next: () => void;
    /**
     * Moves focus onto the previous element.
     */
    unstable_previous: () => void;
    /**
     * Moves focus onto the first element.
     */
    unstable_first: () => void;
    /**
     * Moves focus onto the last element.
     */
    unstable_last: () => void;
    /**
     * Resets `currentId` and `pastId` states.
     */
    unstable_reset: () => void;
    /**
     * Changes the `orientation` state of the roving tab index list.
     */
    unstable_orientate: (orientation: RoverState["orientation"]) => void;
};
export declare type RoverInitialState = Partial<Pick<RoverState, "orientation" | "unstable_currentId" | "unstable_loop">>;
export declare type RoverStateReturn = RoverState & RoverActions;
export declare function useRoverState(initialState?: SealedInitialState<RoverInitialState>): RoverStateReturn;
export declare namespace useRoverState {
    var __keys: ("orientation" | "unstable_stops" | "unstable_currentId" | "unstable_pastId" | "unstable_loop" | "unstable_register" | "unstable_unregister" | "unstable_move" | "unstable_next" | "unstable_previous" | "unstable_first" | "unstable_last" | "unstable_reset" | "unstable_orientate")[];
}
export {};
